require 'libs/shapeData.lua'
require 'libs/class.lua'
require 'libs/vector.lua'

player = class:new()
local playerImg = love.graphics.newImage("art/tempPlayer.png")

function player:init(spawnX, spawnY)
	self.b = love.physics.newBody(world, spawnX, spawnY, 50, 0)
	self.s = love.physics.newCircleShape(self.b, 0, 0, 16)
	self.s:setData(sdata:new())
	self:setData()
	self.b:setMassFromShapes()
	self.s:setCategory(1)
	
	maxJump = 3 --Maximum jumps
	self.canJump = 1 --Can self jump?
	self.canStall = 1 --Can self stall?
	self.canWalk = 0 --Can self walk?
	self.dead = 0
end

function player:setData()
	self.s:getData():setShape(self.s)
	self.s:getData():setBody(self.b)
	self.s:getData():setString("player")
	self.s:getData():setObject(self)
	self.s:getData():setWidth(32)
	self.s:getData():setHeight(32)
end

function player:update(dt)

	--Player Info Updating
	self.X, self.Y = self.b:getPosition()
	self.vel_X, self.vel_Y = self.b:getLinearVelocity()	
	self.angle = self.b:getAngle()
	
	--Movement Limititing
	if self.vel_X > 1000 then
		self.b:setLinearVelocity( 1000, self.vel_Y)
	elseif self.vel_X < -1000 then
		self.b:setLinearVelocity( -1000, self.vel_Y)
	elseif self.vel_Y > 1000 then
		self.b:setLinearVelocity( self.vel_X, 1000)
	elseif self.vel_Y < -1000 then
		self.b:setLinearVelocity( self.vel_X, -1000)
	end
	
	--Overall Screen Movement
	if player_o:getBody():getX() > (resW * .7) + offsetX then
		offsetX = player_o:getBody():getX() - 1024 * .7
	elseif player_o:getBody():getX() < (resW * .3) + offsetX then
		offsetX = player_o:getBody():getX() - 1024 * .3
	end
	
	if player_o:getBody():getY() < (resH * .8) + offsetY then
		offsetY = player_o:getBody():getY() - 1024 * .7
	elseif player_o:getBody():getX() > (resH * .2) + offsetY then
		offsetY = player_o:getBody():getY() - 1024 * .3
	end
	
	pDebug(offsetY)
	
	if offsetX < 0 then
		offsetX = 0
	end
  
	if offsetY < 0 then
		offsetY = 0
	end
	--Controls

		if love.keyboard.isDown("d") then
				self.b:applyForce(3, 0)
		end
		
		if love.keyboard.isDown("a") then
				self.b:applyForce(-3, 0)
		end
		
	
	--Force respawn
	if love.keyboard.isDown("r") then 
		self.dead = 1
	end	
		
	--Jump Code
	if love.keyboard.isDown(" ") then
		self.b:applyImpulse( 0, -.05 )
		
	end
	
	--Death check
	if self.dead == 1 then
		self:respawn( 512,512 )
		self.dead = 0
	end
	
	--DEV TELEPORT (MAKE EDITOR ONLY FOR RELEASE)
	if love.keyboard.isDown("t") then
		self.b:setPosition( mouseX, mouseY )
		pDebug(self.X+offsetX.." , "..self.Y)
	end

end

--Respawn, resets all player traits
function player:respawn( spawnX, spawnY )
	
	self.b:setPosition( spawnX, spawnY )
	self.b:setLinearVelocity( 0,0 )
	self.canJump = 1
	self.canWalk = 0
end

function player:draw()
	love.graphics.draw(playerImg, self.X-offsetX, self.Y-offsetY,  self.angle, 1, 1, 16, 16)
end

function player:getShape()
	return self.s
end

function player:getBody()
	return self.b
end