----------------------------------------------------------------------------
-- LoveUI: A GUI for LVE
--
-- @release unstable 20091312.1112
----------------------------------------------------------------------------
local LoveUI, trigger={}, nil
if type(module)=="function" then
	module (..., package.seeall)
end
----------------------------------------------------------------------------
-- Initialize the LoveUI library
-- @param path The directory containing the LoveUI.lua file
-- @return table
----------------------------------------------------------------------------
function LoveUI:init(path)
	setmetatable(self, self);
	self.name=name
	self.version={"unstable", 20091312.1112};  --release, revision (YYYYMMDD.HHMM)
	self.path=path; --path to folder containing this file
	self.components={} --list of loaded components
	self.services={} --list of loaded functions, e.g functions like love.graphics.draw
	self.images={}
	--load libraries here
	self.class=require(self.path.."/lib/class.lua")
	self.bindings={}
	--set constants here
	self.OFF=0;
	self.ON=1;
	
	trigger=LoveUI:include("trigger")
	--self.compat() -- function to enable backwards compatibility. Doesn't work.
	return self
end

----------------------------------------------------------------------------
-- Get LoveUI's version. Returns build, release
-- @return number, number
----------------------------------------------------------------------------
function LoveUI:version()
	return unpack(self.version)
end

----------------------------------------------------------------------------
-- Loads groups of functions, such as love.graphics, 
-- from the service directory.
-- @param service string
-- @return table
----------------------------------------------------------------------------
function LoveUI:include(service) -- groups of functions, e.g graphics
	if not self[service] then
		self[service]=require(self.path.."/service/"..service..".lua")
		table.insert(self.services, service)
	end
	return self[service]
end

----------------------------------------------------------------------------
-- Loads a component, such as a Button,
-- from the component directory.
-- @param component string
-- @return table
----------------------------------------------------------------------------
function LoveUI:require(component) -- components
	if not self[component] then
		self[component]=require(self.path.."/component/"..component..".lua")
		if type(self[component])=="table" then
			self[component]:load(self)
		else
			error("LoveUI: Component \'"..component.."\' failed to load");
		end
		table.insert(self.components, component)
		self[component].__name=component
		trigger.setup(self[component]);
	end
	return self[component]
end

----------------------------------------------------------------------------
-- Gets an image, such as a Button, using its filename.
-- @param image string
-- @return userdata
----------------------------------------------------------------------------
function LoveUI:getImage(image)
	if not self.images[image] then
		local imagePath=self.path.."/resources/images/"..image
		self.images[image] = LoveUI.graphics.newImage(imagePath)
	end
	return self.images[image]
end



function LoveUI:compat()
	LoveUI.Rect={}
	LoveUI.Size={}
	LoveUI.Point={}
	LoveUI.Stack={}
	function LoveUI.Rect:new(x, y, w, h)
		return {x, y, w, h}
	end
	function LoveUI.Size:new(w, h)
		return {w, h}
	end
	function LoveUI.Point:new(x, y)
		return {x, y}
	end
	function LoveUI.Stack:new(t)
		return t
	end
end


return LoveUI

