----------------------------------------------------------------------------
-- Button. Inherits Control and uses ButtonCell as a backend.
----------------------------------------------------------------------------
local LoveUI, super, Control, ButtonCell, trigger
local Button={}

if type(module)=="function" then
	module (..., package.seeall)
end

----------------------------------------------------------------------------
-- Loads the Button class, automatically called when it's required by
-- LoveUI
-- @param lib table
-- @return class
----------------------------------------------------------------------------
function Button:load(lib)
	LoveUI=lib;
	Control=LoveUI:require("Control");
	ButtonCell=LoveUI:require("ButtonCell");
	trigger=LoveUI:include("trigger");
	super=Control;
	super:subclass(self);
	trigger.setDefault(self, "backgroundColor", "foregroundColor")
	trigger.setDefault(self, "textColor")
	trigger.setDefault(self, "selectColor")
	return self;
end

----------------------------------------------------------------------------
-- Initializes the button objext
-- @param frame table
-- @return Button
----------------------------------------------------------------------------
function Button:init(frame)
	super.init(self, frame,  LoveUI.ButtonCell:new(self, LoveUI:getImage("light-gloss-bottom-top.png")) );
	self.enabled=true;
	self.state=LoveUI.OFF;
	self.value="Button";
	self.cell.alternateImage=LoveUI:getImage("heavy-gloss-top-bottom.png")
	self.opaque=true;
	return self;
end

----------------------------------------------------------------------------
-- This method is invoked by Context whenever it needs to be redrawn.
-- It uses the ButtonCell's display method to do the drawing.
----------------------------------------------------------------------------
function Button:display()
	self.cell:display(self.frame, self);
end

----------------------------------------------------------------------------
-- This method is invoked by Context whenever it needs to be updated.
-- It uses the ButtonCell's update method to do the updating.
----------------------------------------------------------------------------
function Button:update(dt)
	self.cell:update(dt)
end

return Button;