local ButtonCell={}
local LoveUI, ActionCell, super, geometry, graphics, mouse, Control, Event, keyboard
function ButtonCell:load(lib)
	LoveUI=lib;
	ActionCell=LoveUI:require("ActionCell");
	Control=LoveUI:require("Control");
	geometry=LoveUI:include("geometry");
	graphics=LoveUI:include("graphics");
	Event=LoveUI:require("Event");
	keyboard=LoveUI:include("keyboard");
	
	mouse=LoveUI:include("mouse");
	super=ActionCell;
	super:subclass(self)
	return self;
end

function ButtonCell:init(view, image, ...)
	ActionCell.init(self, view, ...);
	self.image=image
	self.mouse_is_down=false
	self.key_is_down=false
	self.alternateImage=self.image;
	
	return self;
end


function ButtonCell:performClick(sender)
	
end

function ButtonCell:drawImage(frame, view)
	if self.controlView.enabled then
		graphics.setColor(self.controlView.backgroundColor)
	else
		graphics.setColor({255, 255, 255})
	end
	local size=geometry.getSize(frame);
	
	if view.opaque then
		graphics.drawBackground(size)
	end
	local curImage=self.image;
	if self.state==LoveUI.ON or self.isdown then
		curImage=self.alternateImage
	end
	graphics.drawBackgroundImage(curImage, {frame[3], frame[4]})
	--graphics.draw(curImage, 0,0,0, frame[3]/self.image:getWidth(), frame[4]/self.image:getHeight());
end



function ButtonCell:update(dt)
	if not mouse.isDown(1) then return end
	-- if mouse moves out then off the button. Might want to move this check to context to save processor.
	local aPoint={mouse.getX(), mouse.getY()};
	local aRect= self.controlView:convertRectToView(self.controlView.scissor);
	if (self.mouse_is_down and self.controlView:mouseInRect(aPoint, aRect)) or self.key_is_down then
		self.state=LoveUI.ON
	else
		self.state=LoveUI.OFF
	end
end

function ButtonCell:display(frame, view)
	self:drawImage(frame, view);
	self:drawTitle(frame, view);
end

function ButtonCell:mouseDown(theEvent)
	if theEvent.button==mouse.mouse_left then
		self.state=LoveUI.ON;
		self.mouse_is_down=true
		super.mouseDown(self, theEvent);
		return true;
	end
	
	return super.mouseDown(self, theEvent);
end

function ButtonCell:mouseUp(theEvent)
	if theEvent.button==mouse.mouse_left then
		if self.state==LoveUI.ON then
			self:activateControlEvent(self.controlView, Event.DEFAULT, theEvent);
		end
		self.state=LoveUI.OFF;
		self.mouse_is_down=false
		super.mouseDown(self, theEvent);
		return true;
	end
	
	return super.mouseDown(self, theEvent);
end

function ButtonCell:keyDown(theEvent)
	if (theEvent.keyCode== keyboard.key_space or theEvent.keyCode== keyboard.key_return) then
		
		--self.color=graphics.newColor(0, 0, 255);
		self.state=LoveUI.ON;
		self.key_is_down=true;
		return true;
	end
	return super.keyDown(self, theEvent);
end

function ButtonCell:keyUp(theEvent)
	if (theEvent.keyCode== keyboard.key_space or theEvent.keyCode== keyboard.key_return) then
		if self.state==LoveUI.ON then
			self:activateControlEvent(self.controlView, LoveUI.EventDefault ,theEvent);
		end
		self.state=LoveUI.OFF;
		self.key_is_down=false;
		return true;
	end
	return super.keyUp(self, theEvent);
end

function ButtonCell:drawTitle(frame, view)
	local curTitle=self.controlView.value;
	graphics.setFont(view.font);
	graphics.setColor(self.controlView.textColor);
	graphics.drawText(self.controlView.value,  frame[3]/2-graphics.getFontWidth(view.font, self.controlView.value)/2, frame[4]/2+graphics.getFontHeight(view.font)/2-1)
end

return ButtonCell;