----------------------------------------------------------------------------
-- Cells act as backends for GUI widgets. e.g Button has ButtonCell that
-- subclasses the ActionCell which subclasses the Cell
----------------------------------------------------------------------------

local Cell={}
local LoveUI, Object, super, trigger

if type(module)=="function" then
	module (..., package.seeall)
end

----------------------------------------------------------------------------
-- Loads the Cell class, automatically called when it's required by
-- LoveUI
-- @param lib table
-- @return class
----------------------------------------------------------------------------
function Cell:load(lib)
	LoveUI=lib;
	Object=LoveUI:require("Object");
	trigger=LoveUI:include("trigger");
	super=Object;
	super:subclass(self);
	trigger.setDefault(self, "font")
	trigger.setDefault(self, "backgroundColor")
	return self;
end

----------------------------------------------------------------------------
-- Initializes the Cell object. 
-- @param value literal
-- @return Cell
----------------------------------------------------------------------------
function Cell:init(value, ...)
	self.value=value;
	self.backgroundColor=nil;
	self.enabled=true;
	self.state=LoveUI.OFF;
	self.font=nil;
	return self;
end

----------------------------------------------------------------------------
-- Overridden by subclasses to manage mouseDown events from the widget
-- @param theEvent Event
----------------------------------------------------------------------------
function Cell:mouseDown(theEvent)
end

----------------------------------------------------------------------------
-- Overridden by subclasses to manage display calls from the widget
-- @param frame Rect
-- @param view View
----------------------------------------------------------------------------
function Cell:display(frame, view)
end

----------------------------------------------------------------------------
-- Overridden by subclasses to manage mouseUp events from the widget
-- @param theEvent Event
----------------------------------------------------------------------------
function Cell:mouseUp(theEvent)
end

----------------------------------------------------------------------------
-- Overridden by subclasses to manage click events
-- @param sender Object
----------------------------------------------------------------------------
function Cell:performClick(sender)
end

return Cell;