----------------------------------------------------------------------------
-- Each instance of Environment stores the graphics environment (such as
-- background color, text color, font, font size, scissor frames, etc) 
-- at the time of the environment's initialization. 
----------------------------------------------------------------------------
local Object, LoveUI, Environment = nil, nil, {};
local graphics=nil;

if type(module)=="function" then
	module (..., package.seeall)
end

----------------------------------------------------------------------------
-- Loads the Environment class, automatically called when it's required by
-- LoveUI
-- @param lib table
-- @return class
----------------------------------------------------------------------------
function Environment:load(lib)
	LoveUI=lib;
	Object=LoveUI:require("Object");
	graphics=LoveUI:include("graphics");
	--keyboard=LoveUI:include("keyboard");
	super=Object;
	super:subclass(self);
	return self;
end

----------------------------------------------------------------------------
-- Initializes the environment object. Stores the current graphics environment
-- @return Environment
----------------------------------------------------------------------------
function Environment:init()
	-- e.g local o=LoveUI.Object:alloc():init();
	self.scissor=nil; --table
	self.color=nil; --table
--	self.textcolor=nil; --table
	self.font=nil; --userdata
	self.linewidth=nil; --number
--	self.pointsize=nil; --number
--	self.keyrepeat=nil;
--	self.linestyle=nil
	self:store();
	return self;
end
--commented out to optimise; commented out parts don't need to be stored....... yet.

----------------------------------------------------------------------------
-- Restores the stored graphics environment
----------------------------------------------------------------------------
function Environment:restore()
	graphics.setScissor(self.scissor);
	if self.font then
		graphics.setFont(self.font);
	end
	graphics.setLineWidth(self.linewidth);
--	graphics.setPointSize(self.pointsize);
	graphics.setColor(self.color);
--	graphics.setTextColor(self.textcolor);
--	graphics.setLineStyle(self.linestyle)
--	keyboard.setKeyRepeat(self.keyrepeat);
		
end

----------------------------------------------------------------------------
-- Stores the current graphics environment
----------------------------------------------------------------------------
function Environment:store()
	self.scissor=graphics.getScissor(); --table
	self.color=graphics.getColor(); --table
--	self.textcolor=graphics.getTextColor(); --table
	self.font=graphics.getFont(); --userdata
	self.linewidth=graphics.getLineWidth(); --number
--	self.pointsize=graphics.getPointSize(); --number
--	self.keyrepeat=keyboard.getKeyRepeat();
--	self.linestyle=graphics.getLineStyle();
end

return Environment;
