--[[
Guide to LoveUI.Textfield.

Properties:
	--Property
		--[Example Values] description
	tabAccessible
		[true/false] whether can select textfield with tab. default true.
	hidden
		[true/false] set false to hide and disable textfield. default false.
	enabled
		[true/false] set false to disable editting. default true.
	textColor
		[love.graphics.newColor(0,0,0)] set to change text color
	opaque
		[true/false] whether to draw blackground default true.
	image
		[nil] set to nil to disable gloss
	value
		["aString"] set this to set string contents
	backgroundColor
		[aColor] set background color
	font
		[aFont] set text font.
		
	setFrame(aFrame)
		to change origin, size of Textfield
		
	selectAll()
		select all. 
	setAction(anAction, EventType, aTarget)
		Refer to LoveUI.lua, search for 'Control Events', possible Event Types are listed there. Not all are responded to. set eventType nil to use default.
]]--

local LoveUI, Control, TextfieldCell, super
local Textfield={}

function Textfield:load(lib)
	LoveUI=lib
	Control=LoveUI:require("Control");
	TextfieldCell=LoveUI:require("TextfieldCell");
	super=Control;
	super:subclass(Textfield);
	return self;
end

function Textfield:init(frame)
	super.init(self, frame, TextfieldCell:new(self, LoveUI:getImage("light-gloss-bottom-top.png")));
	self.editable=true;
	self.enabled=true;
	self.cellClass=LoveUI.TextfieldCell;
	self.cell.value=""
	self.shouldResignFirstResponder=false
	self.opaque=true;
	self.isFirstResponder=false;
	self.justBecameFirstResponder=false
	self.tabAccessible=true;
	return self;
end

function Textfield:display()
	self.cell:display(self.frame, self)
end

function Textfield:update(dt)
	self.cell:update(dt);
end

function Textfield:selectAll()
	self.cell.selectStart=0
	self.cell.selectLength=#tostring(self.cell.value)
end

function Textfield:becomeFirstResponder()
	self.isFirstResponder=true;
	if self.enabled then
		self.cell.state=LoveUI.ON;
		self:selectAll();
		self.justBecameFirstResponder = true;
	end
	return self.enabled;
end

function Textfield:resignFirstResponder()
	return self.cell:resignFirstResponder();
end



return Textfield