local LoveUI, ActionCell, super, keyboard, graphics, geometry, system, mouse, Event
local TextfieldCell={}

function TextfieldCell:load(lib)
	LoveUI=lib
	ActionCell=LoveUI:require("ActionCell")
	Event=LoveUI:require("Event")

	keyboard, graphics =LoveUI:include("keyboard"), LoveUI:include("graphics")
	geometry=LoveUI:include("geometry");
	system=LoveUI:include("system");
	mouse=LoveUI:include("mouse");
	super=ActionCell
	super:subclass(self);
	return self
end

function TextfieldCell:init(view, image, ...)
	-- e.g local o=LoveUI.Object:alloc():init();
	ActionCell.init(self, view, ...);
	self.image=image
	--self.backgroundColor=graphics.newColor(255, 255, 255);
	self.selectStart=0; -- position of selection cursor.
	self.selectLength=0; -- length of selection
	self.offset=0; -- how many pixels is the text shifted to the right.
	self.cushion=6; -- How much room between left edge of textfield to the right edge of the first character
	self.cursorLastShown=0; -- Cursor disappears every 0.5 seconds for 0.5 seconds. This variable records this time.
	self.showCursor=true; -- Whether cursor is seen at this moment in time.
	self.lastMouseDownEvent=nil; -- Last time mouse pressed into text field
	
	return self;
end


			
function TextfieldCell:performClick(sender)
	
end

function TextfieldCell:textHasChanged()
	--Whenever text has changed in the textfield, this function is called.
	--This function then activates the corresponding controlEvent.
	self:activateControlEvent(self.controlView, Event.VALUE_CHANGED, theEvent);
end
function TextfieldCell:deleteBackward(sender)
	if self.selectLength~=0 then
		self:insertText("", self);
		return
	end
	if self.selectStart>0 then
		self.value=string.sub(self.value, 1, self.selectStart-1)..string.sub(self.value, self.selectStart+1)
		self.selectStart=self.selectStart-1
		self:textHasChanged();
	end
end

function TextfieldCell:deleteForward(sender)
	if self.selectLength~=0 then
		self:insertText("", self);
		return
	end
	self.value=string.sub(self.value, 1, self.selectStart)..string.sub(self.value, self.selectStart+2)
	self:textHasChanged();
end

function TextfieldCell:normalize()
	self.value=tostring(self.value)
	if self.selectStart<0 then
		self.selectStart=0
	end
	if self.selectStart>#tostring(self.value) then
		self.selectStart=#tostring(self.value)
	end
	local widthToSelectEnd = self.controlView.font:getWidth(string.sub(tostring(self.value),1,self.selectStart+self.selectLength));
	local widthToSelectStart = self.controlView.font:getWidth(string.sub(tostring(self.value),1,self.selectStart));
	local lastCharX= self.cushion-self.offset+self.controlView.font:getWidth(tostring(self.value))
	local cursorx=widthToSelectEnd - self.offset+self.cushion
	
	if cursorx > self.controlView.frame[3]-self.cushion-1  then --if cursor/edge of select rectangle too far right--try keep text screen full
		self.offset=widthToSelectEnd -self.controlView.frame[3]+self.cushion*1.5
	elseif cursorx < self.cushion then -- if cursor/edge of select rectangle too far left
		self.offset=widthToSelectEnd
	elseif graphics.getWidth(self.controlView.font, tostring(self.value))<self.controlView.frame[3]-self.cushion then
		self.offset=0 
	elseif lastCharX<self.controlView.frame[3]-self.cushion and self.offset>self.cushion then
		self.offset=graphics.getFontWidth(self.controlView.font, tostring(self.value))-self.controlView.frame[3]
	end
end


function TextfieldCell:insertText(text, sender)
	-- Insert the supplied string at the insertion point or selection, deleting the selection if there is one.
	if self.selectLength<0 then
		self.selectStart=self.selectStart+self.selectLength
		self.selectLength=-self.selectLength
	end
	
	self.value=string.sub(self.value, 1, self.selectStart)..text..string.sub(self.value, self.selectStart+self.selectLength+1)
	self.selectStart=self.selectStart+#text

	self.selectLength=0;
	self:textHasChanged();
end

function TextfieldCell:drawImage(frame, view)
	graphics.setColor(self.controlView.backgroundColor)
	local size=geometry.getSize(frame);
	
	if view.opaque then
		graphics.drawBackground(size)
	end
	
	local curImage=self.image;
	if curImage then
		graphics.drawBackgroundImage(curImage, size)
	end
end


function TextfieldCell:resignFirstResponder()
	if not geometry.mouseInRect({mouse.getX(), mouse.getY()}, self.controlView.superview:convertRectToView(self.controlView.frame)) or true then
		self.controlView.shouldResignFirstResponder=true;
	end
	
	if (self.controlView.shouldResignFirstResponder) then
		self.controlView.isFirstResponder=false;
		self.state=LoveUI.OFF;
		self.offset=0
		self.selectStart=0
		self.selectLength=0
	end
	return self.controlView.shouldResignFirstResponder
end


function TextfieldCell:mouseDown(theEvent)
	if theEvent.button==mouse.mouse_right then
		self.selectStart=0
		self.selectLength=#self.value
		return true;
	end
	if theEvent.button==mouse.mouse_left then
		self.controlView.shouldResignFirstResponder=false;
		local mousePos=self.controlView:convertPointFromBase(theEvent.mouseLocation);
		if not geometry.mouseInRect(theEvent.mouseLocation, self.controlView.superview:convertRectToView(self.controlView.frame)) then
			self.controlView.shouldResignFirstResponder=true;
			--repeat mouse click, because the textfield blocked the first mouseclick when resigning first responder was refused.
			self.controlView.context:reclick();
		else
			self.selectStart=self:getTextLocation(self.controlView:convertPointFromBase(theEvent.mouseLocation)) --put cursor to position of mouse click
			self.selectLength=0
			self.lastMouseDownEvent=theEvent
		end
		return true;
	end
	return super.mouseDown(self, theEvent)
end

function TextfieldCell:getTextLocation(aPoint) --get the nth char, at location of x, relative to left edge
	local xValue=aPoint[1]+self.offset
	if xValue <= 0 then
		return 0;
	end
	if xValue >=graphics.getFontWidth(self.controlView.font, self.value) then
		return #tostring(self.value);
	end
	for i = 1, #tostring(self.value), 1 do 
		if (self.controlView.font:getWidth(string.sub(self.value,0,i))>xValue) then
		return i-1
		end 
	end
	return 0
end

function TextfieldCell:mouseUp(theEvent)
	if theEvent.button==mouse.mouse_left and theEvent.clickCount>=2 then
		self.controlView:selectAll()
		return true
	end
	return super.mouseUp(self, theEvent)
end

function TextfieldCell:keyDown(theEvent)
	--check code range
	local key=theEvent.keyCode;
	if key > 31 and key < 127 then
		local char = string.char( key )
		if keyboard.isDown( keyboard.key["lshift"] ) or keyboard.isDown( keyboard.key["rshift"] ) then
			
			--char = string.upper( char )
			char=keyboard.key_upper[char];
		end
		self:insertText(char, self)
		self:normalize()
		return
	elseif key == keyboard.key_tab then
		--self:insertText("	", self)
	elseif key == keyboard.key_backspace then
		self:deleteBackward(self);
		self:normalize()
	elseif key == keyboard.key_delete then
		self:deleteForward(self);
		return self:normalize()
	elseif key == keyboard.key_left then
		if keyboard.isDown(keyboard.key["lshift"]) or  keyboard.isDown( keyboard.key["rshift"] ) then
			if self.selectStart+self.selectLength>0 then
				self.selectLength=self.selectLength-1
			end
		else -- if not shift selecting  
			if self.selectLength==0 and self.selectStart>0 then
				self.selectStart=self.selectStart-1
			elseif self.selectLength > 0 then
				self.selectLength=0
			else
				self.selectStart=self.selectStart+self.selectLength
				self.selectLength=0
			end
		end
		return self:normalize()
	elseif key == keyboard.key_right then
		if keyboard.isDown(keyboard.key["lshift"]) or  keyboard.isDown(keyboard.key["rshift"] ) then
			if self.selectStart+self.selectLength<#tostring(self.value) then
				self.selectLength=self.selectLength+1
			end
		else -- if not shift selecting 
			if self.selectLength==0 and self.selectStart<#tostring(self.value) then
				self.selectStart=self.selectStart+1
			elseif self.selectLength > 0 then
				self.selectStart=self.selectStart+1+self.selectLength
				self.selectLength=0
			else
				self.selectLength=0;
			end
		end
		return self:normalize()
	elseif key == keyboard.key_home then
		if keyboard.isDown(keyboard.key["lshift"]) or  keyboard.isDown( keyboard.key["rshift"] ) then
			self.selectLength = -self.selectStart
		else -- if no select rectangle
			self.selectStart=0
			self.selectLength=0
		end
		return self:normalize()
	elseif key == keyboard.key_end then
		if keyboard.isDown(keyboard.key["lshift"]) or  keyboard.isDown( keyboard.key["rshift"] ) then
			self.selectLength = #tostring(self.value) - self.selectStart
		else -- if no select rectangle
			self.selectStart=#tostring(self.value)
			self.selectLength=0
		end
		return self:normalize()
	end
	return super.keyDown(self, theEvent)
end
function TextfieldCell:display(frame, view)
	self:drawImage(frame, view);
	self:drawText(frame, view);
	self:drawSelection(frame, view)
	self:drawBorder(frame, view)
end

function TextfieldCell:update(dt)
	self.showCursor=true;
	if system.time()-self.cursorLastShown > 1000 then
		self.showCursor=true
		self.cursorLastShown=system.time();
	elseif system.time()-self.cursorLastShown>500 then
		self.showCursor=false;
	end
	
	if (mouse.isDown(mouse.mouse_left) and self.lastMouseDownEvent~=nil) and self.controlView.isFirstResponder then --dragging the rectangle
		local mousePoint=self.controlView:convertPointFromBase({mouse.getX(), mouse.getY()})
		if self.controlView:convertPointFromBase(self.lastMouseDownEvent.mouseLocation).x ~= mousePoint[1] then
			self.selectLength=self:getTextLocation(mousePoint)-self.selectStart;
		end
		self:normalize()
	end
end

function TextfieldCell:drawBorder(frame, view)
	graphics.setColor({180,180,180,255})
	graphics.setLineStyle( graphics.line_rough )
	--graphics.rectangle(love.draw_line, 0,0, frame.size.width-1, frame.size.height-1) --textfield border
	graphics.drawRectangle({1, 0, frame[3]-1, frame[4]-1})
	graphics.setColor({0,0,0,255})
	graphics.setLineStyle( graphics.line_smooth )
	
	graphics.drawLine({0, 1}, {frame[3], 1}) --text field shadow at top
end

function TextfieldCell:drawText(frame, view)
	self.value=tostring(self.value);
	graphics.setFont(view.font);
	graphics.setColor(self.controlView.textColor);
	graphics.drawText(self.value, self.cushion-self.offset, frame[4]/2+5)
end

function TextfieldCell:drawSelection(frame, view)
	self.value=tostring(self.value);
	
		graphics.setLineStyle( graphics.line_rough )
		local xloc=graphics.getFontWidth(view.font, string.sub(self.value, 1, self.selectStart))+self.cushion-self.offset
		if self.selectLength==0 then
			if self.showCursor and view.isFirstResponder then
				graphics.setColor(self.controlView.textColor)
				graphics.drawLine({xloc, 3}, {xloc ,frame[4]-3})
			end
			
		else
			graphics.setColor(self.controlView.selectColor) --select rect color
			if self.selectLength>0 then
				graphics.drawFilledRectangle({xloc, 3, graphics.getFontWidth(view.font, string.sub(self.value,self.selectStart+1,self.selectStart+self.selectLength)) ,frame[4]-6})
			else
				local selWid=graphics.getFontWidth(view.font, string.sub(self.value,self.selectStart+self.selectLength+1, self.selectStart))
				graphics.drawFilledRectangle({xloc-selWid, 3, selWid ,frame[4]-5})
			end
		end
	
end

return TextfieldCell