----------------------------------------------------------------------------
-- A module of geometric functions
----------------------------------------------------------------------------
local geometry={}
if type(module)=="function" then
	module (..., package.seeall)
end
----------------------------------------------------------------------------
-- Given a rectangle in the format {x, y, width, height}, returns its origin
-- @param rect table
-- @return table
----------------------------------------------------------------------------
function geometry.getOrigin(rect)
	return {rect[1], rect[2]}
end

----------------------------------------------------------------------------
-- Given a rectangle in the format {x, y, width, height}, returns its size
-- @param rect table
-- @return table
----------------------------------------------------------------------------
function geometry.getSize(rect)
	return {rect[3], rect[4]}
end


----------------------------------------------------------------------------
-- Check whether every value in array1 is equal to every value in array2. 
-- Used for equating rectangles
-- @param array1 table
-- @param array2 table
-- @return boolean
----------------------------------------------------------------------------
function geometry.equals(array1, array2)
	for k, v in pairs(array1) do
		if array2[k]~=v then
			return false;
		end
	end
	return true;
end

function geometry.add(array1, array2)
	local result={}
	for k, v in pairs(array1) do
		result[k]=v+array2[k]
	end
	return result;
end

function geometry.subtract(array1, array2)
	local result={}
	for k, v in pairs(array1) do
		result[k]=v-array2[k]
	end
	return result;
end

function geometry.mouseInRect(aPoint, aRect)
	return 
		aPoint[1] >= aRect[1] and
		aPoint[2] >= aRect[2] and
		aPoint[1] <= aRect[1] + aRect[3] and
		aPoint[2] <= aRect[2] + aRect[4];	
end

function geometry.pointInRect(aPoint, aRect)
	return 
		aPoint[1] > aRect[1] and
		aPoint[2] > aRect[2] and
		aPoint[1] < aRect[1] + aRect[3] and
		aPoint[2] < aRect[2] + aRect[4];	
end
	
return geometry