require 'game.lua'
require 'menu.lua'
require 'entities/entList.lua'
require 'libs/mapList.lua'
require 'libs/shapeData.lua'
require 'libs/class.lua'
require 'libs/vector.lua'

state = {} --this is what gets called into action, whether that be the game, level editor, whatever...

function love.load()

	--Initialize some variables
	debugTime = 0
	curTime = 0
	printdebug = 0

	--initial graphics setup
	love.graphics.setBackgroundColor(80, 120, 220)
	love.graphics.setMode(1024, 1024, false, true, 0)

	state = menu:new()

end

function love.update(dt)

	--Info gathering
	curTime = love.timer.getTime()
	mouseX = love.mouse.getX()
	mouseY = love.mouse.getY()
	
	if curTime - debugTime >= 1 then
		printdebug = 0
	end
	
	if state.update then
		state:update(dt)
	end
	
end

function love.draw()

	--pDebug
	if printdebug == 1 then
		love.graphics.print(ptext.."\n", debugX, debugY)
	end
	
	if state.draw then
		state:draw()
	end
end

function love.keypressed(key, unicode)
	if state.keypressed then
		state:keypressed(key, unicode)
	end
end

function love.keyreleased(key)
	if state.keyreleased then
		state:keyreleased(key)
	end
end

function love.mousepressed(x, y, button)
	if state.mousepressed then
		state:mousepressed(x, y, button)
	end
end

function love.mousereleased(x, y, button)
	if state.mousereleased then
		state:mousereleased(x, y, button)
	end
end

--Special Physics Calls
function add(a, b, coll)
end

function persist(a, b, coll)
end

function rem(a, b, coll)
end

function pDebug(text, dX, dY)

	ptext = text
	debugX = dX or 100
	debugY = dY or 100	
	printdebug = 1
	debugTime = love.timer.getTime()	
	
end
function getClosest()

	checkX = mouseX
	checkY = mouseY
	closestDist = math.huge

	if closestObject and math.sqrt( (closestObject.X - checkX)^2 + (closestObject.Y - checkY)^2 ) > closestDist then
		closestDist = math.huge
	end
	
	for j = 1, terrainChunk.blockNumW + 1 do
		for i = 1, terrainChunk.blockNumH + 1 do
			if not terrainChunk.Squares[i][j].destroyed then
				local curDist = math.sqrt( (terrainChunk.Squares[i][j].X-offsetX-checkX)^2+(terrainChunk.Squares[i][j].Y-offsetY-checkY)^2)
			
				if curDist < closestDist then
					closestDist = curDist
					closestObject = terrainChunk.Squares[i][j]
					pDebug(curDist)
				end
			end
		end
	end
	
end